@@ -34,22 +34,32 @@
 (defvar gnus-pick-mode nil
   "Minor mode for providing a pick-and-read interface in Gnus summary buffers.")
 
-(defvar gnus-pick-display-summary nil
-  "*Display summary while reading.")
+(defcustom gnus-pick-display-summary nil
+  "*Display summary while reading."
+  :type 'boolean
+  :group 'gnus-summary-pick)
 
-(defvar gnus-pick-mode-hook nil
-  "Hook run in summary pick mode buffers.")
+(defcustom gnus-pick-mode-hook nil
+  "Hook run in summary pick mode buffers."
+  :type 'hook
+  :group 'gnus-summary-pick)
 
-(defvar gnus-mark-unpicked-articles-as-read nil
-  "*If non-nil, mark all unpicked articles as read.")
+(defcustom gnus-mark-unpicked-articles-as-read nil
+  "*If non-nil, mark all unpicked articles as read."
+  :type 'boolean
+  :group 'gnus-summary-pick)
 
-(defvar gnus-pick-elegant-flow t
-  "If non-nil, gnus-pick-start-reading will run gnus-summary-next-group when no articles have been picked.")
+(defcustom gnus-pick-elegant-flow t
+  "If non-nil, gnus-pick-start-reading will run gnus-summary-next-group when no articles have been picked."
+  :type 'boolean
+  :group 'gnus-summary-pick)
 
-(defvar gnus-summary-pick-line-format
+(defcustom gnus-summary-pick-line-format
   "%-5P %U\%R\%z\%I\%(%[%4L: %-20,20n%]%) %s\n"
   "*The format specification of the lines in pick buffers.
-It accepts the same format specs that `gnus-summary-line-format' does.")
+It accepts the same format specs that `gnus-summary-line-format' does."
+  :type 'string
+  :group 'gnus-summary-pick)
 
 ;;; Internal variables.
 
@@ -133,7 +143,7 @@ It accepts the same format specs that `gnus-summary-line-format' does.")
 	     (save-excursion
 	       (set-buffer gnus-summary-buffer)
 	       gnus-pick-mode))
-    (message-add-action 
+    (message-add-action
      '(gnus-configure-windows 'pick t) 'send 'exit 'postpone 'kill)))
 
 (defvar gnus-pick-line-number 1)
@@ -153,12 +163,12 @@ If given a prefix, mark all unpicked articles as read."
         (when (or catch-up gnus-mark-unpicked-articles-as-read)
 	  (gnus-summary-limit-mark-excluded-as-read))
         (gnus-summary-first-article)
-        (gnus-configure-windows 
+        (gnus-configure-windows
 	 (if gnus-pick-display-summary 'article 'pick) t))
     (if gnus-pick-elegant-flow
 	(progn
 	  (when (or catch-up gnus-mark-unpicked-articles-as-read)
-	    (gnus-summary-limit-mark-excluded-as-read))
+	    (gnus-summary-catchup nil t))
 	  (if (gnus-group-quit-config gnus-newsgroup-name)
 	      (gnus-summary-exit)
 	    (gnus-summary-next-group)))
@@ -315,7 +325,7 @@ This must be bound to a button-down mouse event."
   (interactive "P")
   (when (eq major-mode 'gnus-summary-mode)
     (make-local-variable 'gnus-binary-mode)
-    (setq gnus-binary-mode 
+    (setq gnus-binary-mode
 	  (if (null arg) (not gnus-binary-mode)
 	    (> (prefix-numeric-value arg) 0)))
     (when gnus-binary-mode
@@ -350,16 +360,22 @@ This must be bound to a button-down mouse event."
 ;;; gnus-tree-mode
 ;;;
 
-(defvar gnus-tree-line-format "%(%[%3,3n%]%)"
-  "Format of tree elements.")
+(defcustom gnus-tree-line-format "%(%[%3,3n%]%)"
+  "Format of tree elements."
+  :type 'string
+  :group 'gnus-summary-tree)
 
-(defvar gnus-tree-minimize-window t
+(defcustom gnus-tree-minimize-window t
   "If non-nil, minimize the tree buffer window.
 If a number, never let the tree buffer grow taller than that number of
-lines.")
+lines."
+  :type 'boolean
+  :group 'gnus-summary-tree)
 
-(defvar gnus-selected-tree-face 'modeline
-  "*Face used for highlighting selected articles in the thread tree.")
+(defcustom gnus-selected-tree-face 'modeline
+  "*Face used for highlighting selected articles in the thread tree."
+  :type 'face
+  :group 'gnus-summary-tree)
 
 (defvar gnus-tree-brackets '((?\[ . ?\]) (?\( . ?\))
 			     (?\{ . ?\}) (?< . ?>))
@@ -368,20 +384,28 @@ lines.")
 (defvar gnus-tree-parent-child-edges '(?- ?\\ ?|)
   "Characters used to connect parents with children.")
 
-(defvar gnus-tree-mode-line-format "Gnus: %%b %S %Z"
-  "*The format specification for the tree mode line.")
+(defcustom gnus-tree-mode-line-format "Gnus: %%b %S %Z"
+  "*The format specification for the tree mode line."
+  :type 'string
+  :group 'gnus-summary-tree)
 
-(defvar gnus-generate-tree-function 'gnus-generate-vertical-tree
+(defcustom gnus-generate-tree-function 'gnus-generate-vertical-tree
   "*Function for generating a thread tree.
 Two predefined functions are available:
-`gnus-generate-horizontal-tree' and `gnus-generate-vertical-tree'.")
+`gnus-generate-horizontal-tree' and `gnus-generate-vertical-tree'."
+  :type '(radio (function-item gnus-generate-vertical-tree)
+		(function-item gnus-generate-horizontal-tree)
+		(function :tag "Other" nil))
+  :group 'gnus-summary-tree)
 
-(defvar gnus-tree-mode-hook nil
-  "*Hook run in tree mode buffers.")
+(defcustom gnus-tree-mode-hook nil
+  "*Hook run in tree mode buffers."
+  :type 'hook
+  :group 'gnus-summary-tree)
 
 ;;; Internal variables.
 
-(defvar gnus-tree-line-format-alist 
+(defvar gnus-tree-line-format-alist
   `((?n gnus-tmp-name ?s)
     (?f gnus-tmp-from ?s)
     (?N gnus-tmp-number ?d)
@@ -410,6 +434,7 @@ Two predefined functions are available:
    "\r" gnus-tree-select-article
    gnus-mouse-2 gnus-tree-pick-article
    "\C-?" gnus-tree-read-summary-keys
+   "h" gnus-tree-show-summary
 
    "\C-c\C-i" gnus-info-find-node)
 
@@ -426,11 +451,11 @@ Two predefined functions are available:
 (defun gnus-tree-mode ()
   "Major mode for displaying thread trees."
   (interactive)
-  (setq gnus-tree-mode-line-format-spec 
-	(gnus-parse-format gnus-tree-mode-line-format 
+  (setq gnus-tree-mode-line-format-spec
+	(gnus-parse-format gnus-tree-mode-line-format
 			   gnus-summary-mode-line-format-alist))
-  (setq gnus-tree-line-format-spec 
-	(gnus-parse-format gnus-tree-line-format 
+  (setq gnus-tree-line-format-spec
+	(gnus-parse-format gnus-tree-line-format
 			   gnus-tree-line-format-alist t))
   (when (gnus-visual-p 'tree-menu 'menu)
     (gnus-tree-make-menu-bar))
@@ -460,6 +485,14 @@ Two predefined functions are available:
 	(goto-char (or (gnus-overlay-end gnus-selected-tree-overlay) 1)))
       (gnus-tree-minimize))))
 
+(defun gnus-tree-show-summary ()
+  "Reconfigure windows to show summary buffer."
+  (interactive)
+  (if (not (gnus-buffer-live-p gnus-summary-buffer))
+      (error "There is no summary buffer for this tree buffer")
+    (gnus-configure-windows 'article)
+    (gnus-summary-goto-subject gnus-current-article)))
+
 (defun gnus-tree-select-article (article)
   "Select the article under point, if any."
   (interactive (list (gnus-tree-article-number)))
@@ -509,7 +542,7 @@ Two predefined functions are available:
 	;; possible valid number, or the second line from the top,
 	;; whichever is the least.
 	(set-window-start
-	 tree-window (min bottom (save-excursion 
+	 tree-window (min bottom (save-excursion
 				   (forward-line (- top)) (point)))))
       (select-window selected))))
 
@@ -528,7 +561,7 @@ Two predefined functions are available:
     (let ((windows 0)
 	  tot-win-height)
       (walk-windows (lambda (window) (incf windows)))
-      (setq tot-win-height 
+      (setq tot-win-height
 	    (- (frame-height)
 	       (* window-min-height (1- windows))
 	       2))
@@ -613,8 +646,8 @@ Two predefined functions are available:
 		    (not (eval (caar list))))
 	  (setq list (cdr list)))))
     (unless (eq (setq face (cdar list)) (get-text-property beg 'face))
-      (gnus-put-text-property 
-       beg end 'face 
+      (gnus-put-text-property
+       beg end 'face
        (if (boundp face) (symbol-value face) face)))))
 
 (defun gnus-tree-indent (level)
@@ -646,7 +679,9 @@ Two predefined functions are available:
   "Generate a horizontal tree."
   (let* ((dummy (stringp (car thread)))
 	 (do (or dummy
-		 (memq (mail-header-number (car thread)) gnus-tmp-limit)))
+		 (and (car thread)
+		      (memq (mail-header-number (car thread))
+			    gnus-tmp-limit))))
 	 col beg)
     (if (not do)
 	;; We don't want this article.
@@ -718,13 +753,12 @@ Two predefined functions are available:
 	  (delete-char -1)
 	  (insert (cadr gnus-tree-parent-child-edges))
 	  (setq beg (point))
+	  (forward-char -1)
 	  ;; Draw "-" lines leftwards.
-	  (while (progn
-		   (unless (bolp)
-		     (forward-char -2))
-		   (= (following-char) ? ))
-	    (delete-char 1)
-	    (insert (car gnus-tree-parent-child-edges)))
+	  (while (= (char-after (1- (point))) ? )
+	    (delete-char -1)
+	    (insert (car gnus-tree-parent-child-edges))
+	    (forward-char -1))
 	  (goto-char beg)
 	  (gnus-tree-forward-line 1)))
       (setq dummyp nil)
@@ -757,8 +791,8 @@ Two predefined functions are available:
       (let ((top (save-excursion
 		   (set-buffer gnus-summary-buffer)
 		   (gnus-cut-thread
-		    (gnus-remove-thread 
-		     (mail-header-id 
+		    (gnus-remove-thread
+		     (mail-header-id
 		      (gnus-summary-article-header article))
 		     t))))
 	    (gnus-tmp-limit gnus-newsgroup-limit)
@@ -788,7 +822,7 @@ Two predefined functions are available:
 	 (setq gnus-selected-tree-overlay (gnus-make-overlay 1 2))
 	 'face gnus-selected-tree-face))
       ;; Move the overlay to the article.
-      (gnus-move-overlay 
+      (gnus-move-overlay
        gnus-selected-tree-overlay (goto-char (car region)) (cdr region))
       (gnus-tree-minimize)
       (gnus-tree-recenter)
@@ -809,7 +843,7 @@ Two predefined functions are available:
     (let (region)
       (when (setq region (gnus-tree-article-region article))
 	(gnus-put-text-property (car region) (cdr region) 'face face)
-	(set-window-point 
+	(set-window-point
 	 (get-buffer-window (current-buffer) t) (cdr region))))))
 
 ;;;
@@ -841,7 +875,7 @@ Two predefined functions are available:
     ("exit" . gnus-group-exit)))
 
 (defvar gnus-carpal-summary-buffer-buttons
-  '("mark" 
+  '("mark"
     ("read" . gnus-summary-mark-as-read-forward)
     ("tick" . gnus-summary-tick-article-forward)
     ("clear" . gnus-summary-clear-mark-forward)
@@ -874,7 +908,7 @@ Two predefined functions are available:
     ("exit" . gnus-summary-exit)
     ("fed-up" . gnus-summary-catchup-and-goto-next-group)))
 
-(defvar gnus-carpal-server-buffer-buttons 
+(defvar gnus-carpal-server-buffer-buttons
   '(("add" . gnus-server-add-server)
     ("browse" . gnus-server-browse-server)
     ("list" . gnus-server-list-servers)
@@ -941,10 +975,10 @@ The following commands are available:
       (save-excursion
 	(set-buffer (get-buffer-create buffer))
 	(gnus-carpal-mode)
-	(setq gnus-carpal-attached-buffer 
+	(setq gnus-carpal-attached-buffer
 	      (intern (format "gnus-%s-buffer" type)))
 	(gnus-add-current-to-buffer-list)
-	(let ((buttons (symbol-value 
+	(let ((buttons (symbol-value
 			(intern (format "gnus-carpal-%s-buffer-buttons"
 					type))))
 	      (buffer-read-only nil)
